local VERSION = tonumber(("$Revision: 42202 $"):match("%d+"))

local _G = getfenv(0)

local L = AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Info")
local BC = AceLibrary("Babble-Class-2.2")
local deformat = AceLibrary("Deformat-2.0")

local AuldLangSyne = AuldLangSyne
local Info = AuldLangSyne:NewModule("Info", "AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0")
if AuldLangSyne.revision < VERSION then
	AuldLangSyne.version = "r" .. VERSION
	AuldLangSyne.revision = VERSION
	AuldLangSyne.date = ("$Date: 2007-06-29 04:56:24 -0400 (Fri, 29 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

function Info:OnInitialize(name)
	self.db = AuldLangSyne:AcquireDBNamespace("Info")
	AuldLangSyne:RegisterDefaults("Info", "profile", {
		pvp = true,
		pvpnum = false,
		guild = true,
		color = {
			name = true,
			class = true,
		},
		compact = false,
		clevels = true,
	})
	
	self.menu = {
		handler = Info,
		type = "group",
		name = L["Show"], desc = L["Choose what information to display"],
		args = {
			pvp = {
				name = "PVP", type = "toggle",
				desc = L["Turns display of PVP ranks on and off."],
				get = function() return self.db.profile.pvp end,
				set = function(t) self.db.profile.pvp = t end,
			},
			pvpnum = {
				name = L["PVP rank numbers"], type = "toggle",
				desc = L["Turns display of PVP rank numbers on and off."],
				get = function() return self.db.profile.pvpnum end,
				set = function(t) self.db.profile.pvpnum = t end,
			},
			guild = {
				name = L["Guild"], type = "toggle",
				desc = L["Turns display of guild membership on and off."],
				get = function() return self.db.profile.guild end,
				set = function(t) self.db.profile.guild = t end,
			},
			color = {
				type = "group", name = L["Colors"], desc = L["Choose which elements to color"],
				args = {
					name = {
						name = L["Name"], type = "toggle",
						desc = L["Turns coloring names by gender on and off."],
						get = function() return self.db.profile.color.name end,
						set = function(t) self.db.profile.color.name = t end,
					},
					class = {
						name = L["Class"], type = "toggle",
						desc = L["Turns coloring class names on and off."],
						get = function() return self.db.profile.color.class end,
						set = function(t) self.db.profile.color.class = t end,
					}
				},
			},
			compact = {
				name = L["Compact mode"], type = "toggle",
				desc = L["Turns compact friends list on and off."],
				get = function() return self.db.profile.compact end,
				set = function(t) self.db.profile.compact = t end,
			},
			clevels = {
				name = L["Compact mode levels"], type = "toggle",
				desc = L["Turns display of levels in compact mode on and off."],
				get = function() return self.db.profile.clevels end,
				set = function(t) self.db.profile.clevels = t end,
			},
		},
	}
	
	self.colors_sex = {
		[MALE] = "90b0f0",
		[FEMALE] = "f090c4",
	}
	
	self.faction = AuldLangSyne.faction
	
	if AuldLangSyne.db.realm.chars then
		for name, data in pairs(AuldLangSyne.db.realm.chars) do
			self.db.realm[name] = data
		end
		AuldLangSyne.db.realm.chars = nil
	end
end

function Info:OnEnable(first)
	self:RegisterEvent("FRIENDLIST_UPDATE")
	self:SecureHook("FriendsList_Update")
	self:SecureHook(GameTooltip, "SetUnit")
	self:Hook("GetWhoInfo", true)
	self:SecureHook("SendWho")
end

--function Info:OnDisable()
--	
--end

------------------------------------------------------------------------
-- Friend info display
------------------------------------------------------------------------

function Info:FRIENDLIST_UPDATE()
	local numFriends = GetNumFriends()

	for i=1, numFriends do
		local name, level, class, area, connected, status = GetFriendInfo(i)

		if name and connected then
			if self.db.realm[name] == nil then self.db.realm[name] = {} end
			self.db.realm[name].level = level
			self.db.realm[name].class = class
			self.db.realm[name].updated = time()
		end
	end
end

function Info:FriendsList_Update()
	if FriendsFrame:IsVisible() and FriendsFrame.selectedTab == 1 then
		local friendOffset = FauxScrollFrame_GetOffset(FriendsFrameFriendsScrollFrame)
		local numFriends = GetNumFriends()
		local n = FRIENDS_TO_DISPLAY

		if numFriends < FRIENDS_TO_DISPLAY then n = numFriends end

		for i=1, n, 1 do
			local name, level, class, area, connected, status = GetFriendInfo(i + friendOffset)

			-- Do we know anything about them?
			if self.db.realm[name] and self.db.realm[name].level then
				local nameLocationText = _G["FriendsFrameFriendButton"..(i).."ButtonTextNameLocation"]
				local infoText = _G["FriendsFrameFriendButton"..(i).."ButtonTextInfo"]

				local race = self.db.realm[name].race and (self.db.realm[name].race.." ") or ""
				local rank = (self.db.realm[name].rank and self.db.realm[name].rank ~= 0) and ((self.db.profile.pvpnum and ("[R".. tonumber(self.db.realm[name].rank)-4 .."]") or self:GetRankName(self.db.realm[name].rank, self.db.realm[name].sex)).." ") or ""
				local lastseen = ""
				if not connected then
					level = self.db.realm[name].level
					class = self.db.realm[name].class
					lastseen = connected and nil or (self.db.realm[name].updated and math.ceil((time() - self.db.realm[name].updated)/86400))
					if connected and nil or self.db.realm[name].updated then
						lastseen = self:HowLongAgo(self.db.realm[name].updated)
					end
				end

				if self.db.profile.compact then
					-- Right now offline friends that have never been seen by the addon look out of place, for they are not getting munged.  Fix?
					local pvprank = self.db.realm[name].rank
					if not pvprank then
						pvprank = "?"
					else
						pvprank = tonumber(pvprank)-4
					end
					local nlt = (connected and "" or "|cff999999")..(self.db.profile.color.class and ("|cff"..BC:GetHexColor(class)) or "")..name..(self.db.profile.color.class and "|r" or "")..((self.db.profile.color.name and self.db.realm[name].sex) and "|r" or "")..(connected and "" or "|cff999999")..(self.db.profile.pvp and (" / R"..pvprank) or "")..(self.db.profile.clevels and (" / L"..self.db.realm[name].level) or "")..((self.db.profile.guild and self.db.realm[name].guild) and (" / "..self.db.realm[name].guild) or "")..(connected and (" / "..area..((status ~= "") and " ("..status..")" or "")) or (lastseen and (" / "..lastseen)))
					local it = "|cff999999"..((self.db.realm[name].note and (self.db.realm[name].note ~= "")) and self.db.realm[name].note or "")

					nameLocationText:SetText(nlt)
					infoText:SetText(it)
				else
					local nlt = ((self.db.profile.color.name and self.db.realm[name].sex) and ("|cff"..self.colors_sex[self.db.realm[name].sex]) or "")..(self.db.profile.pvp and rank or "")..name..((self.db.profile.color.name and self.db.realm[name].sex) and "|r" or "")..(connected and "" or ("|cff999999 - "..lastseen..L[" ago"]))
					local it = race..(self.db.profile.color.class and ("|cff"..BC:GetHexColor(class)) or "")..class..(self.db.profile.color.class and "|r" or "")..(connected and "" or "|cff999999")..((self.db.profile.guild and self.db.realm[name].guild) and (" <"..self.db.realm[name].guild..">") or "")

					if connected then
						nameLocationText:SetText(format(FRIENDS_LIST_TEMPLATE, nlt, area, status))
					else
						nameLocationText:SetText(format(FRIENDS_LIST_OFFLINE_TEMPLATE, nlt))
					end
					infoText:SetText(format(FRIENDS_LEVEL_TEMPLATE, level, it))
				end
				-- Make sure we're not overflowing
				if nameLocationText:GetWidth() > 275 then
					nameLocationText:SetJustifyH("LEFT")
					nameLocationText:SetWidth(275)
					nameLocationText:SetHeight(14) -- If we don't set the height, the line wraps.
				end
				if infoText:GetWidth() > 275 then
					infoText:SetJustifyH("LEFT")
					infoText:SetWidth(275)
					infoText:SetHeight(14) -- If we don't set the height, the line wraps.
				end
			end
		end
	end
end

function Info:SetUnit(object, unitid)
	if UnitIsPlayer(unitid) then
		local name = UnitName(unitid)
		if self.db.realm[name] ~= nil then
			self.db.realm[name].rank = UnitPVPRank(unitid)
			self.db.realm[name].race = UnitRace(unitid)

			local sex = UnitSex(unitid)
			-- 1: neuter/unknown, 2: male, 3: female
			-- We're not storing the output of UnitSex directly, for Blizzard has been known to change it.
			if sex == 2 then
				self.db.realm[name].sex = MALE
			elseif sex == 3 then
				self.db.realm[name].sex = FEMALE
			end

			local guildName, guildRankName, guildRankIndex = GetGuildInfo(unitid)
			self.db.realm[name].guild = guildName
			self.db.realm[name].guildRank = guildRankName

			self.db.realm[name].updated = time()

			if FriendsFrame:IsVisible() and FriendsFrame.selectedTab == 1 then
				FriendsList_Update()
			end
		end
	end
end

function Info:GetWhoInfo(i)
	local name, guild, level, race, class, zone, group = self.hooks["GetWhoInfo"](i)
	if self.db.realm[name] ~= nil then
		if guild ~= "" then self.db.realm[name].guild = guild end
		self.db.realm[name].level = level
		self.db.realm[name].race = race
		self.db.realm[name].class = class
		self.db.realm[name].updated = time()

		if FriendsFrame:IsVisible() and FriendsFrame.selectedTab == 1 then
			FriendsList_Update()
		end
	end
	return name, guild, level, race, class, zone, group
end

do
	local function stopListening()
		Info:UnregisterEvent("CHAT_MSG_SYSTEM")
	end
	function Info:SendWho(filter)
		self:RegisterEvent("CHAT_MSG_SYSTEM")
		self:ScheduleEvent("ALSInfo_stoplistening", stopListening, 0.5)
	end
end

function Info:CHAT_MSG_SYSTEM(message)
	local _, name, level, race, class, guild, location = deformat(message, WHO_LIST_GUILD_FORMAT)
	if not name then _, name, level, race, class, location = deformat(message, WHO_LIST_GUILD_FORMAT) end
	if name and self.db.realm[name] then
		if guild then self.db.realm[name].guild = guild end
		self.db.realm[name].level = level
		self.db.realm[name].race = race
		self.db.realm[name].class = class
		self.db.realm[name].updated = time()
	end
end

------------------------------------------------------------------------
-- Utility functions
------------------------------------------------------------------------

function Info:HowLongAgo(ago)
	if ago then
		local currentTime = time()
		local minutes = math.ceil((currentTime - ago) / 60)
		if minutes > 59 then
			local hours = math.ceil((currentTime - ago) / 3600)
			if hours > 23 then
				local days = math.ceil((currentTime - ago) / 86400)
				lastseen = days..L[" day"]..((days > 1) and L["s"] or "")
			else
				lastseen = hours..L[" hour"]..((hours > 1) and L["s"] or "")
			end
		else
			lastseen = minutes..L[" minute"]..((minutes > 1) and L["s"] or "")
		end
		return lastseen
	else
		return ""
	end
end

function Info:GetRankName(rank, sex)
	--"rank" should be the number of the PVP rank (starts at 5, goes to 19).
	--"sex" should be either MALE or FEMALE.
	return _G["PVP_RANK_"..rank.."_"..self.faction..((sex == FEMALE) and "_FEMALE" or "")]
end